@@ -7,7 +7,7 @@
 
   This module requires at least python 2.2
 
-:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2000-2009 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 :license: General Public License version 2 - http://www.gnu.org/licenses
 """
@@ -15,13 +15,13 @@ __docformat__ = "restructuredtext en"
 
 from types import ListType, TupleType
 
-class Interface:
+class Interface(object):
     """Base class for interfaces."""
     def is_implemented_by(cls, instance):
         return implements(instance, cls)
     is_implemented_by = classmethod(is_implemented_by)
 
-    
+
 def implements(obj, interface):
     """Return true if the give object (maybe an instance or class) implements
     the interface.
@@ -39,7 +39,7 @@ def extend(klass, interface, _recurs=False):
     """Add interface to klass'__implements__ if not already implemented in.
 
     If klass is subclassed, ensure subclasses __implements__ it as well.
-    
+
     NOTE: klass should be e new class.
     """
     if not implements(klass, interface):
